#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import argparse
import sys


def print_includes():
    from ._dev import get_fwdpp_includes
    from ._dev import get_includes

    idirs = [get_includes(), get_fwdpp_includes()]

    print(' '.join('-I' + idir for idir in idirs))


def get_fwdpy11_includes():
    from ._dev import get_includes
    return get_includes()


def get_fwdpp_includes():
    from ._dev import get_fwdpp_includes
    return get_fwdpp_includes()


def print_mako():
    from ._dev import minimal_mako
    print(minimal_mako())


def main():
    parser = argparse.ArgumentParser(prog='python -m fwdpy11')
    parser.add_argument('--includes', action='store_true',
                        help='Print the CPPFLAGS required to use fwdpy11 and fwdpp headers.')
    parser.add_argument('--fwdpy11_headers', action='store_true',
                        help='Get location of fwdpy11 headers')
    parser.add_argument('--fwdpp_headers', action='store_true',
                        help='Get location of fwdpp headers')
    parser.add_argument('--mako', action='store_true',
                        help="Print minimal mako header for use with cppimport.")
    args = parser.parse_args()
    if not sys.argv[1:]:
        parser.print_help()
    if args.includes:
        print_includes()
        return
    if args.fwdpy11_headers:
        print(get_fwdpy11_includes())
        return
    if args.fwdpp_headers:
        print(get_fwdpp_includes())
        return
    if args.mako:
        print_mako()
        return


if __name__ == "__main__":
    main()



import unittest
import numpy as np
import fwdpy11


class testSampleRecorder(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.sr = fwdpy11.SampleRecorder()

    def test_add_sample(self):
        self.sr.add_sample(1)
        self.assertEqual(len(self.sr.samples), 1)
        self.assertEqual(self.sr.samples[0], 1)

    def test_assign(self):
        samples = np.array([1, 2, 3, 4], dtype=np.uint32)
        self.sr.assign(samples)
        self.assertEqual(len(self.sr.samples), 4)
        for i in range(4):
            self.assertEqual(self.sr.samples[i], i+1)


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11
import numpy as np
from quick_pops import quick_nonneutral_slocus


class TestDataMatrixFromDiploidPopulation(unittest.TestCase):
    """
    Much of this is already tested in fwdpp's unit tests,
    but the purpose here is to make sure that the wrappers
    are working appropriately.
    """
    @classmethod
    def setUpClass(self):
        self.pop = quick_nonneutral_slocus()
        self.indlist = [i for i in range(100, 150)]
        self.keys = fwdpy11.mutation_keys(self.pop, self.indlist)
        self.hm = fwdpy11.haplotype_matrix(
            self.pop, self.indlist, self.keys[0], self.keys[1])
        self.gm = fwdpy11.genotype_matrix(
            self.pop, self.indlist, self.keys[0], self.keys[1])
        self.hm_neutral = np.array(self.hm.neutral)
        self.hm_selected = np.array(self.hm.selected)
        self.gm_neutral = np.array(self.gm.neutral)
        self.gm_selected = np.array(self.gm.selected)

    def testNcol(self):
        self.assertEqual(self.hm.ncol, 2*len(self.indlist))
        self.assertEqual(self.gm.ncol, len(self.indlist))

    def testKeyNeutralityAndCount(self):
        for i in self.keys[0]:
            self.assertTrue(self.pop.mutations[i[0]].neutral)
            self.assertTrue(self.pop.mcounts[i[0]] > 0)
        for i in self.keys[1]:
            self.assertFalse(self.pop.mutations[i[0]].neutral)
            self.assertTrue(self.pop.mcounts[i[0]] > 0)

    def testHapMat(self):
        self.assertTrue(self.hm_neutral.dtype == np.int8)
        self.assertEqual(self.hm_neutral.ndim, 2)
        self.assertTrue(self.hm_selected.dtype == np.int8)
        self.assertEqual(self.hm_selected.ndim, 2)
        # Get the col sums
        colSums = self.hm_neutral.sum(axis=0)
        colSumsSel = self.hm_selected.sum(axis=0)
        self.assertEqual(len(colSums), self.hm_neutral.shape[1])
        self.assertEqual(len(colSumsSel), self.hm_selected.shape[1])
        j = 0
        for i in range(100, 150):
            # Num neutral variants in diploid i, gamete 0
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].first].mutations)
            self.assertEqual(nmuts, colSums[j])
            # Num neutral variants in diploid i, gamete 1
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].second].mutations)
            self.assertEqual(nmuts, colSums[j + 1])

            # Now, test numbers of selected
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].first].smutations)
            self.assertEqual(nmuts, colSumsSel[j])
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].second].smutations)
            self.assertEqual(nmuts, colSumsSel[j + 1])

            j += 2

    def testGenoMat(self):
        self.assertTrue(self.gm_neutral.dtype == np.int8)
        self.assertEqual(self.gm_neutral.ndim, 2)
        # Get the col sums
        colSums = self.gm_neutral.sum(axis=0)
        colSumsSel = self.gm_selected.sum(axis=0)
        self.assertEqual(len(colSums), self.gm_neutral.shape[1])
        self.assertEqual(len(colSumsSel), self.gm_selected.shape[1])
        j = 0
        for i in range(100, 150):
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].first].mutations)
            nmuts += len(
                self.pop.haploid_genomes[self.pop.diploids[i].second].mutations)
            self.assertEqual(nmuts, colSums[j])

            # Now, test numbers of selected
            nmuts = len(
                self.pop.haploid_genomes[self.pop.diploids[i].first].smutations)
            nmuts += len(
                self.pop.haploid_genomes[self.pop.diploids[i].second].smutations)
            self.assertEqual(nmuts, colSumsSel[j])

            j += 1

    def testHapMatToSample(self):
        neut_sample, sel_sample = fwdpy11.matrix_to_sample(self.hm)
        rowSums = self.hm_neutral.sum(axis=1)
        rowSumsSel = self.hm_selected.sum(axis=1)
        self.assertEqual(len(neut_sample), self.hm_neutral.shape[0])
        self.assertEqual(len(rowSums), len(neut_sample))
        self.assertEqual(len(sel_sample), self.hm_selected.shape[0])
        self.assertEqual(len(rowSumsSel), len(sel_sample))
        i = 0
        for ni in neut_sample:
            num_ones = ni[1].count('1')
            self.assertEqual(num_ones, rowSums[i])
            i += 1

    def test_merge(self):
        m, keys = self.gm.merge()
        s = (self.gm_neutral.shape[0] + self.gm_selected.shape[0],
             self.gm_neutral.shape[1])

        self.assertEqual(m.shape, s)

        neutral = np.array(
            [self.pop.mutations[k].neutral for k in keys], dtype=np.int32)
        w = np.where(neutral == 1)[0]
        npos = self.gm.neutral.positions
        npos_idx = np.argsort(npos)
        self.assertTrue(np.array_equal(m[w, :], self.gm_neutral[npos_idx, :]))
        self.assertTrue(np.array_equal(self.gm.neutral_matrix[0], m[w, :]))


if __name__ == "__main__":
    unittest.main()



import unittest
import numpy_array_interface


class TestNumpyArrayInterface1D(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.v = numpy_array_interface.VecBackedArray()

    def test_readwrite(self):
        a = self.v.x_readwrite()
        self.assertEqual(a.flags.owndata, False)
        self.assertEqual(a.flags.writeable, True)

    def test_readonly(self):
        a = self.v.x_readonly()
        self.assertEqual(a.flags.owndata, False)
        self.assertEqual(a.flags.writeable, False)

    def test_readwrite_via_capsule(self):
        a = self.v.x_readwrite_via_capsule()
        self.assertEqual(a.flags.owndata, False)
        self.assertEqual(a.flags.writeable, True)
        b = self.v.x_readwrite()
        self.assertEqual(b.shape[0], 0)


class TestNumpyArrayInterface2D(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.v = numpy_array_interface.VecBacked2DArray(7, 3)

    def test_shape(self):
        a = self.v.x_readwrite()
        self.assertEqual(a.shape, self.v.shape)

    def test_readwrite(self):
        a = self.v.x_readwrite()
        self.assertEqual(a.flags.owndata, False)
        self.assertEqual(a.flags.writeable, True)

    def test_readonly(self):
        a = self.v.x_readonly()
        self.assertEqual(a.flags.owndata, False)
        self.assertEqual(a.flags.writeable, False)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2020 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import numpy as np


def _merge(self):
    """
    Merge the neutral and selected data.

    :rtype: tuple
    :returns: a :class:`numpy.ndarray` composed of
              neutral and selected mutations.  Rows (mutations)
              are sorted by position.  The second tuple element
              contains the mutation keys.

    .. versionadded:: 0.6.1
    """
    nneutral = len(self.neutral_keys)
    nselected = len(self.selected_keys)

    if nneutral == 0 and nselected > 0:
        return np.array(self.selected), self.selected_keys
    elif nneutral > 0 and nselected == 0:
        return np.array(self.neutral), self.neutral_keys

    positions = np.concatenate((self.neutral.positions,
                                self.selected.positions))
    sorted_pos_indexes = np.argsort(positions)
    merged = np.concatenate(
        (np.array(self.neutral), np.array(self.selected)), axis=0)
    keys = np.concatenate((self.neutral_keys, self.selected_keys))
    return merged[sorted_pos_indexes, :], keys[sorted_pos_indexes]

# NOTE: the following two properties only need sorting if
# the DataMatrix is not generated from a tree sequence.


def _return_sorted_matrix(self, sm, k):
    pos = sm.positions
    posi = np.argsort(pos)
    return np.array(sm, copy=False)[posi, :], k


def _sorted_neutral(self):
    """
    :returns: Neutral mutations and keys
    :rtype: tuple

    The neutral matrix is a :class:`numpy.ndarray`

    .. versionadded:: 0.6.1
    """
    return _return_sorted_matrix(self, self.neutral, self.neutral_keys)


def _sorted_selected(self):
    """
    :returns: Selected mutations and keys
    :rtype: tuple

    The selected matrix is a :class:`numpy.ndarray`

    .. versionadded:: 0.6.1
    """
    return _return_sorted_matrix(self, self.selected, self.selected_keys)


def _patch_data_matrix(dm):
    dm.merge = _merge
    dm.neutral_matrix = property(_sorted_neutral)
    dm.selected_matrix = property(_sorted_selected)



import unittest
import fwdpy11
import numpy as np


class testAdditive(unittest.TestCase):
    @classmethod
    def setUp(self):
        GN = fwdpy11.GaussianNoise
        self.w = fwdpy11.Additive(2.0)
        self.t = fwdpy11.Additive(
            2.0, fwdpy11.GSS(0.0, 1.0))
        self.tn = fwdpy11.Additive(1.0,
                                   fwdpy11.GSS(
                                       0.0, 1.0),
                                   GN(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))

    def testPickleTraitWithNoiseToFile(self):
        import pickle
        with open("ptest.pickle", "wb") as f:
            pickle.dump(self.tn, f)

        with open("ptest.pickle", "rb") as f:
            up = pickle.load(f)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testMultiplicative(unittest.TestCase):
    @classmethod
    def setUp(self):
        GN = fwdpy11.GaussianNoise
        self.w = fwdpy11.Multiplicative(2.0)
        self.t = fwdpy11.Multiplicative(
            2.0, fwdpy11.GSS(0.0, 1.0))
        self.tn = fwdpy11.Multiplicative(1.0,
                                         fwdpy11.GSS(
                                             0.0, 1.0),
                                         GN(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testGBR(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.gss = fwdpy11.GSS(0.0, 1.0)
        self.gnoise = fwdpy11.GaussianNoise(
            mean=0.0, sd=1.0)
        self.nonoise = fwdpy11.NoNoise()

    def testPicklingGSS(self):
        import pickle
        gbr = fwdpy11.GBR(self.gss)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.nonoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))

    def testPicklingGSSGaussianNoise(self):
        import pickle
        gbr = fwdpy11.GBR(self.gss, self.gnoise)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.gnoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))


class testGSS(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.x = fwdpy11.GSS(0.0, 1.0)

    def testProperties(self):
        self.assertEqual(self.x.opt, 0.0)
        self.assertEqual(self.x.VS, 1.0)


class testGSSandGSSmoConsistency(unittest.TestCase):
    """
    This tests that GSS and GSSmo have opt and VS
    in the same order.  If that were not true,
    fitness calculations would come out differently
    and the test would fail.
    """
    @classmethod
    def setUp(self):
        self.a = fwdpy11.Additive(
            2.0, fwdpy11.GSS(0.0, 1.0))
        self.b = fwdpy11.Additive(
            2.0, fwdpy11.GSSmo([(0, 0.0, 1.0)]))
        self.pop = fwdpy11.DiploidPopulation(1000)

    def testFitnesses(self):
        wa = [self.a.fitness(i, self.pop) for i in range(self.pop.N)]
        wb = [self.b.fitness(i, self.pop) for i in range(self.pop.N)]
        for i, j in zip(wa, wb):
            self.assertEqual(i, j)


if __name__ == "__main__":
    unittest.main()



import fwdpy11
import numpy as np
import unittest
import poptools
import testMultivariateGSSmo


def set_up_quant_trait_model():
    N = 1000
    demography = np.array([N]*10*N, dtype=np.uint32)
    rho = 1000.
    r = rho/(4*N)

    timepoints = np.array([0], dtype=np.uint32)
    ntraits = 4
    optima = np.array(np.zeros(len(timepoints)*ntraits))
    optima = optima.reshape((len(timepoints), ntraits))
    GSSmo = fwdpy11.MultivariateGSSmo(timepoints, optima, 1)
    cmat = np.identity(ntraits)
    np.fill_diagonal(cmat, 0.1)
    a = fwdpy11.StrictAdditiveMultivariateEffects(ntraits, 0, GSSmo)
    p = {'nregions': [],
         'sregions': [fwdpy11.MultivariateGaussianEffects(0, 1, 1, cmat)],
         'recregions': [fwdpy11.Region(0, 1, 1)],
         'rates': (0.0, 0.001, r),
         'gvalue': a,
         'prune_selected': False,
         'demography': demography
         }
    params = fwdpy11.ModelParams(**p)
    rng = fwdpy11.GSLrng(101*45*110*210)
    pop = fwdpy11.DiploidPopulation(N, 1.0)
    return params, rng, pop, ntraits


class TestMultivariateGSSmo(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.params, self.rng, self.pop, self.ntraits = set_up_quant_trait_model()

    def test_github_issue_310(self):
        fwdpy11.evolvets(self.rng, self.pop, self.params, 100)
        nonmutants = 0
        md = np.array(self.pop.diploid_metadata, copy=False)
        for i, j in enumerate(self.pop.diploids):
            n0 = len(self.pop.haploid_genomes[j.first].smutations)
            n1 = len(self.pop.haploid_genomes[j.second].smutations)
            if md['w'][i] != 1.:
                self.assertTrue(n0 + n1 > 0)
            elif n1+n0 == 0:
                nonmutants += 1

            if nonmutants == 0:
                self.fail(
                    "Test invalid: zero individuals were mutation-free")


class TestMultivariateGSSmoCPP(unittest.TestCase):
    """
    Tests of C++ API
    """
    def setUp(self):
        self.pop = fwdpy11.DiploidPopulation(100, 1.0)
        self.ntraits = 4

    def test_multiple_optima(self):
        timepoints = np.array([0, 5*self.pop.N])
        optima = np.array(np.zeros(len(timepoints)*self.ntraits))
        optima = optima.reshape((len(timepoints), self.ntraits))
        optima[1, 0] = 1
        GSSmo = fwdpy11.MultivariateGSSmo(timepoints, optima, 1)

        co = testMultivariateGSSmo.get_optima(GSSmo)
        self.assertTrue(all([i == 0 for i in co]))
        poptools.change_generation(self.pop, timepoints[1])
        testMultivariateGSSmo.update(self.pop, GSSmo)
        co = testMultivariateGSSmo.get_optima(GSSmo)
        self.assertEqual(co[0], 1)
        self.assertTrue(all([i == 0 for i in co[1:]]))

        poptools.change_generation(self.pop, timepoints[1]+10)
        testMultivariateGSSmo.update(self.pop, GSSmo)
        co2 = testMultivariateGSSmo.get_optima(GSSmo)

        self.assertTrue(co == co2)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2020 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import fwdpy11
import unittest
import numpy as np

# NOTE: these tests can all be improved in the future
# by having a conversion from msprime/tskit that lifts
# over mutations assuming that they are neutral.
# With that in place, we can compare to the msprime/tskit
# genotype matrix, which makes for a more independent test
# of correctness.


def fs_from_ndarray(gm):
    gm_rc = np.sum(gm, axis=1)
    gm_uc = np.unique(gm_rc, return_counts=True)
    gm_fs = np.zeros(gm.shape[1]+1, dtype=np.int32)
    gm_fs[gm_uc[0]] += gm_uc[1]
    return gm_fs


class TestSingleDemeCase(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        import msprime
        Ne = 1000
        Nr = 100.0
        ts = msprime.simulate(2*Ne, Ne=Ne, recombination_rate=Nr/Ne,
                              random_seed=666)
        self.pop = fwdpy11.DiploidPopulation.create_from_tskit(ts)
        rng = fwdpy11.GSLrng(12343)
        fwdpy11.infinite_sites(rng, self.pop, Nr/Ne)

    def test_comparison_to_genotype_matrix(self):
        dm = fwdpy11.data_matrix_from_tables(self.pop.tables,
                                             self.pop.alive_nodes, True, False)
        gm = np.array(dm.neutral, copy=False)
        gm_fs = fs_from_ndarray(gm)
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes])
        self.assertTrue(np.array_equal(gm_fs[1:-1], tc_fs.data[1:-1]))
        self.assertTrue(np.array_equal(gm_fs, tc_fs.data))

    def test_comparison_to_genotype_matrix_for_sample(self):
        A, B = 103, 210
        dm = fwdpy11.data_matrix_from_tables(self.pop.tables,
                                             self.pop.alive_nodes[A:B],
                                             True, False)
        gm = np.array(dm.neutral, copy=False)
        gm_fs = fs_from_ndarray(gm)
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes[A:B]])
        self.assertTrue(np.array_equal(gm_fs[1:-1], tc_fs.data[1:-1]))

    def test_skipping_neutral_variants(self):
        tc_fs = self.pop.tables.fs(
            [self.pop.alive_nodes], include_neutral=False)
        self.assertEqual(tc_fs.sum(), 0)

    def test_empty_samples_list(self):
        with self.assertRaises(ValueError):
            self.pop.tables.fs([])

    def test_nodes_out_of_range(self):
        with self.assertRaises(ValueError):
            samples = np.array([len(self.pop.tables.nodes)], dtype=np.int32)
            self.pop.tables.fs([samples])

    def test_two_overlapping_windows(self):
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes])
        wfs = self.pop.tables.fs([self.pop.alive_nodes],
                                 windows=[(0, 1./3.), (1./3., 1.0)])
        self.assertTrue(np.array_equal(wfs, tc_fs))

    def test_three_overlapping_windows(self):
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes])
        wfs = self.pop.tables.fs([self.pop.alive_nodes],
                                 windows=[(0, 1./3.), (1./3., 2./3.),
                                          (2./3., 1)])
        self.assertTrue(np.array_equal(wfs, tc_fs))

    def test_random_number_of_windows(self):
        nw = np.random.randint(100, 300)
        lefts = np.arange(nw)/(nw-1)
        windows = [(lefts[i], lefts[i+1]) for i in range(len(lefts)-1)]
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes])
        wfs = self.pop.tables.fs([self.pop.alive_nodes],
                                 windows=windows)
        self.assertTrue(np.array_equal(wfs, tc_fs))

    def test_separated_windows(self):
        windows = [(0, 0.25), (0.66, 0.9)]
        dm = fwdpy11.data_matrix_from_tables(self.pop.tables,
                                             self.pop.alive_nodes,
                                             True, False)
        gm = np.array(dm.neutral, copy=False)
        gm_pos = np.array([self.pop.mutations[k].pos for k in dm.neutral_keys])
        gm_pos_in_windows = np.where((gm_pos < 0.25) |
                                     ((gm_pos >= 0.66) & (gm_pos < 0.9)))[0]
        gm = gm[gm_pos_in_windows, :]
        gm_fs = fs_from_ndarray(gm)
        tc_fs = self.pop.tables.fs([self.pop.alive_nodes], windows=windows)
        self.assertTrue(np.array_equal(gm_fs, tc_fs))


class TestTwoDemeCase(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        import msprime
        Ne = 1000
        Nr = 100.0
        nodes_per_deme = 1000
        config = [msprime.PopulationConfiguration(nodes_per_deme),
                  msprime.PopulationConfiguration(nodes_per_deme)]

        events = [msprime.MassMigration(1*Ne, 1, 0, 1.0)]
        ts = msprime.simulate(population_configurations=config,
                              demographic_events=events,
                              Ne=Ne, recombination_rate=Nr/Ne,
                              random_seed=666)
        self.pop = fwdpy11.DiploidPopulation.create_from_tskit(ts)
        rng = fwdpy11.GSLrng(12343)
        fwdpy11.infinite_sites(rng, self.pop, Nr/Ne)

    def test_marginal_deme_fs(self):
        a = self.pop.alive_nodes
        nodes = np.array(self.pop.tables.nodes, copy=False)
        d0 = a[np.where(nodes['deme'][a] == 0)[0]]
        d1 = a[np.where(nodes['deme'][a] == 1)[0]]

        for samples in (d0, d1):
            tc_fs = self.pop.tables.fs([samples])
            dm = fwdpy11.data_matrix_from_tables(self.pop.tables,
                                                 samples, True, True)
            gm = np.array(dm.neutral, copy=False)
            gm_fs = fs_from_ndarray(gm)
            self.assertTrue(np.array_equal(gm_fs[1:-1], tc_fs.data[1:-1]))

    def test_joint_deme_fs(self):
        a = self.pop.alive_nodes
        nodes = np.array(self.pop.tables.nodes, copy=False)
        d0 = a[np.where(nodes['deme'][a] == 0)[0]]
        d1 = a[np.where(nodes['deme'][a] == 1)[0]]

        tc_fs = self.pop.tables.fs([d0, d1])
        tc_fs0 = tc_fs.sum(axis=1).todense()
        tc_fs1 = tc_fs.sum(axis=0).todense()
        for i, j in zip((tc_fs0, tc_fs1), (d0, d1)):
            dm = fwdpy11.data_matrix_from_tables(self.pop.tables,
                                                 j, True, True)
            gm = np.array(dm.neutral, copy=False)
            gm_fs = fs_from_ndarray(gm)
            self.assertTrue(np.array_equal(gm_fs[1:-1], i.data[1:-1]))

    def test_joint_deme_fs_marginalize(self):
        a = self.pop.alive_nodes
        nodes = np.array(self.pop.tables.nodes, copy=False)
        d0 = a[np.where(nodes['deme'][a] == 0)[0]]
        d1 = a[np.where(nodes['deme'][a] == 1)[0]]
        tc_fs = self.pop.tables.fs([d0, d1])
        tc_fs_deme0 = tc_fs.sum(axis=1).todense()
        tc_fs_deme1 = tc_fs.sum(axis=0).todense()

        tc_fs2 = self.pop.tables.fs([d0, d1], marginalize=True)
        self.assertTrue(np.array_equal(tc_fs_deme0, tc_fs2[0]))
        self.assertTrue(np.array_equal(tc_fs_deme1, tc_fs2[1]))

    def test_marginalizing_to_three_samples(self):
        a = self.pop.alive_nodes
        sample_lists = [a[:10], a[20:30], a[200:210]]
        fs1 = self.pop.tables.fs([sample_lists[0]])
        fs2 = self.pop.tables.fs([sample_lists[1]])
        fs3 = self.pop.tables.fs([sample_lists[2]])

        mfs = self.pop.tables.fs(sample_lists, marginalize=True)

        for i, j in zip(range(3), [fs1, fs2, fs3]):
            self.assertTrue(np.ma.allequal(mfs[i], j))

    def test_marginalizing_to_three_samples_in_separate_windows(self):
        a = self.pop.alive_nodes
        sample_lists = [a[:10], a[20:30], a[200:210]]
        windows = [(0.1, 0.23), (0.723, 0.85)]
        fs1 = self.pop.tables.fs(
            [sample_lists[0]], windows=windows, separate_windows=True)
        fs2 = self.pop.tables.fs(
            [sample_lists[1]], windows=windows, separate_windows=True)
        fs3 = self.pop.tables.fs(
            [sample_lists[2]], windows=windows, separate_windows=True)

        mfs = self.pop.tables.fs(sample_lists, marginalize=True,
                                 windows=windows, separate_windows=True)

        for i, j in zip(mfs, range(len(windows))):
            self.assertTrue(np.ma.allequal(i[0], fs1[j]))
            self.assertTrue(np.ma.allequal(i[1], fs2[j]))
            self.assertTrue(np.ma.allequal(i[2], fs3[j]))


if __name__ == "__main__":
    unittest.main()
